(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.CloudinaryTinyJs = factory());
}(this, (function () { 'use strict';

var invariantFor = function invariantFor(type) {
  return function (condition, parameter, value, message) {
    var source = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : '/image_transformation_reference#' + parameter + '_parameter';

    if (!condition) {
      value = typeof value === 'string' ? '\'' + value + '\'' : value;
      source = source ? 'http://cloudinary.com/documentation' + source : null;
      throw new Error('Cloudinary ' + type + ':: ' + parameter + ' ' + message + ', received: ' + value + (source ? ' - see ' + source : ''));
    }
  };
};

var invariant = invariantFor('');
var invariantImage = invariantFor('Image ');


var isNumber = function isNumber(value) {
  return typeof value === 'number' || !Number.isNaN(Number(value));
};

var startsWith = function startsWith(needle, haystack) {
  return haystack.indexOf(needle) === 0;
};

var includes = function includes(needle, haystack) {
  return haystack.indexOf(needle) > -1;
};

var shouldBeOneOf = function shouldBeOneOf(possibleValues) {
  return 'should be one of [\'' + possibleValues.join('\', \'') + '\']';
};

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var typeOptions = ['upload', 'fetch', 'facebook', 'twitter', 'twitter_name', 'instagram', 'instagram_name', 'gplus', 'gravatar'];

var compile = function compile(parameterSet, transform, defaultTransform) {
  if (!transform || !parameterSet || Object.keys(transform).length === 0) {
    return '';
  }

  var compile = function compile(parameters) {
    return Object.keys(parameters).map(function (param) {
      return parameterSet(param, parameters[param]);
    }).join(',');
  };

  return '/' + (Array.isArray(transform) ? transform.map(compile).join('/') : compile(_extends({}, defaultTransform, transform)));
};

var urlBuilder = function urlBuilder() {
  var parameterSets = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var defaultResourceType = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'image';
  return function (_ref) {
    var cloudName = _ref.cloudName,
        _ref$cdnSubdomain = _ref.cdnSubdomain,
        cdnSubdomain = _ref$cdnSubdomain === undefined ? false : _ref$cdnSubdomain,
        _ref$cname = _ref.cname,
        cname = _ref$cname === undefined ? 'res.cloudinary.com' : _ref$cname,
        _ref$secure = _ref.secure,
        defaultSecure = _ref$secure === undefined ? true : _ref$secure,
        _ref$defaults = _ref.defaults;
    _ref$defaults = _ref$defaults === undefined ? {} : _ref$defaults;

    var _ref$defaults$type = _ref$defaults.type,
        defaultType = _ref$defaults$type === undefined ? 'upload' : _ref$defaults$type,
        defaultTransform = _objectWithoutProperties(_ref$defaults, ['type']);

    "development" !== 'production' && invariant(cloudName, 'cloudName', cloudName, 'configuration is required', '/node_additional_topics#configuration_options');

    var baseUrl = cname + '/' + cloudName + '/';
    var sub = '';

    return function (publicId) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var _options$resourceType = options.resourceType,
          resourceType = _options$resourceType === undefined ? defaultResourceType : _options$resourceType,
          _options$secure = options.secure,
          secure = _options$secure === undefined ? defaultSecure : _options$secure,
          _options$type = options.type,
          type = _options$type === undefined ? defaultType : _options$type,
          _options$version = options.version,
          version = _options$version === undefined ? '1' : _options$version,
          transform = _objectWithoutProperties(options, ['resourceType', 'secure', 'type', 'version']);

      if (options.transform) {
        transform = options.transform;
      } else if (Array.isArray(options)) {
        transform = options;
      }

      "development" !== 'production' && invariant(!transform || Object.keys(transform).length === 0 || resourceType === 'raw' || includes(resourceType, Object.keys(parameterSets)), 'resourceType', resourceType, shouldBeOneOf(['raw'].concat(Object.keys(parameterSets))) + ', fix the resource type or add additional transform parameters to the configuration', null);

      "development" !== 'production' && invariant(includes(type, typeOptions), 'type', type, shouldBeOneOf(typeOptions), null);

      "development" !== 'production' && invariant(!cdnSubdomain || typeof cdnSubdomain === 'function', 'cdnSubdomain', cdnSubdomain, 'should be a CRC function: `(string) => number`', null);

      if (cdnSubdomain) {
        sub = 'a' + (cdnSubdomain(publicId) % 5 + 1) + '.';
      }

      transform = compile(parameterSets[resourceType], transform, defaultTransform);

      return 'http' + (secure ? 's' : '') + '://' + sub + baseUrl + resourceType + '/' + type + transform + '/v' + version + '/' + publicId;
    };
  };
};

// http://cloudinary.com/documentation/image_transformation_reference#format_parameter
var formatOptions = ['jpg', 'jpe', 'jpeg', 'jpc', 'jp2', 'j2k', 'wdp', 'jxr', 'hdp', 'png', 'gif', 'webp', 'bmp', 'tif', 'tiff', 'ico', 'pdf', 'ps', 'ept', 'eps', 'eps3', 'psd', 'svg', 'ai', 'djvu', 'flif'];

// http://cloudinary.com/documentation/image_transformation_reference#crop_parameter
var cropOptions = ['scale', 'fit', 'limit', 'mfit', 'fill', 'lfill', 'pad', 'lpad', 'mpad', 'crop', 'thumb', 'imagga_crop', 'imagga_scale'];

// http://cloudinary.com/documentation/image_transformation_reference#gravity_parameter
var gravityOptions = ['north_west', 'north', 'north_east', 'west', 'center', 'east', 'south_west', 'south', 'south_east', 'xy_center', 'face', 'face:center', 'face:auto', 'faces', 'faces:center', 'faces:auto', 'body', 'body:face', 'ocr_text', 'adv_face', 'adv_faces', 'adv_eyes', 'custom', 'custom:face', 'custom:faces', 'custom:adv_face', 'custom:adv_faces'];

// http://cloudinary.com/documentation/image_transformation_reference#quality_parameter
var qualityOptions = ['auto', 'auto:best', 'auto:good', 'auto:eco', 'auto:low', 'jpegmini'];

// http://cloudinary.com/documentation/image_transformation_reference#angle_parameter
var angleOptions = ['auto_right', 'auto_left', 'ignore', 'vflip', 'hflip'];

// http://cloudinary.com/documentation/image_transformation_reference#background_parameter
var backgroundOptions = ['auto:border', 'auto:predominant', 'auto:border_contrast', 'auto:predominant_contrast'];

// http://cloudinary.com/documentation/image_transformation_reference#color_parameter
// A color name or rgb: followed by an rgb[a] or rrggbb[aa] hex value
var colorRegex = /^(?:[a-z]+$|rgb:(?:[0-9a-f]{3,4}$|[0-9a-f]{6}$|[0-9a-f]{8}$))/i;

// http://cloudinary.com/documentation/image_transformation_reference#color_space_parameter
var colorSpaceOptions = ['srgb', 'tinysrgb', 'no_cmyk'];

var flagOptions = ['any_format', 'attachment', 'apng', 'awebp', 'clip', 'clip_evenodd', 'cutter', 'force_strip', 'ignore_aspect_ratio', 'immutable_cache', 'keep_attribution', 'keep_iptc', 'layer_apply', 'lossy', 'no_overflow', 'preserve_transparency', 'png8', 'png24', 'png32', 'progressive', 'progressive:semi', 'progressive:steep', 'progressive:none', 'rasterize', 'region_relative', 'relative', 'strip_profile', 'text_no_trim', 'tiff8_lwz', 'tiled'];

var validate = {
  width: function width(value) {
    return invariantImage(isNumber(value) || value === 'auto' || startsWith('auto:', value), 'width', value, 'should be a number, \'auto\', or a string starting with \'auto:\'');
  },
  height: function height(value) {
    return invariantImage(isNumber(value), 'height', value, 'should be a number');
  },
  crop: function crop(value) {
    return invariantImage(includes(value, cropOptions), 'crop', value, shouldBeOneOf(cropOptions));
  },
  aspectRatio: function aspectRatio(value) {
    return invariantImage(isNumber(value) || value.match(/^\d+:\d+$/), 'aspectRatio', value, 'should be a number or have the form x:y');
  },
  gravity: function gravity(value) {
    return invariantImage(includes(value, gravityOptions) || value === 'auto' || startsWith('auto:', value), 'gravity', value, shouldBeOneOf(gravityOptions) + ', \'auto\', or a string starting with \'auto:\'');
  },
  zoom: function zoom(value) {
    return invariantImage(isNumber(value), 'zoom', value, 'should be a number');
  },
  x: function x(value) {
    return invariantImage(isNumber(value), 'x', value, 'should be a number');
  },
  y: function y(value) {
    return invariantImage(isNumber(value), 'y', value, 'should be a number');
  },
  fetchFormat: function fetchFormat(value) {
    return invariantImage(includes(value, ['auto'].concat(formatOptions)), 'fetchFormat', value, shouldBeOneOf(['auto'].concat(formatOptions)) + ' or \'auto\'');
  },
  quality: function quality(value) {
    return invariantImage(isNumber(value) && 1 <= +value && +value <= 100 || typeof value === 'string' && value.match(/^\d+:\d+$/) || includes(value, qualityOptions), 'quality', value, shouldBeOneOf(formatOptions) + ', a number between 1 and 100, or have the form x:y');
  },
  radius: function radius(value) {
    return invariantImage(isNumber(value) || value === 'max' || value.match(/^\d+(?::\d+){0,3}$/), 'radius', value, 'should be a number, \'max\' or have the form x[:y[:z[:u]]]');
  },
  angle: function angle(value) {
    return invariantImage(isNumber(value) || includes(value, angleOptions), 'angle', value, shouldBeOneOf(angleOptions) + ' or a number');
  },
  effect: function effect(value) {},
  opacity: function opacity(value) {
    return invariantImage(isNumber(value) && 0 <= +value && +value <= 100, 'opacity', value, 'should be a number between 0 and 100');
  },
  border: function border(value) {
    return invariantImage(value.match(new RegExp('^\\d+px_solid_' + colorRegex.toString().replace(/[/^i]/g, '') + '$', 'i')), 'border', value, 'should be an object with \'width\' & \'color\' or a string of the form \'width_style_color\'');
  },
  borderObjectColor: function borderObjectColor(value) {
    return invariantImage('color' in value, 'border', JSON.stringify(value), 'should contain a \'color\' property when provided as an object');
  },
  background: function background(value) {
    return invariantImage(typeof value === 'string' && (includes(value, backgroundOptions) || value.match(colorRegex)), 'background', value, shouldBeOneOf(backgroundOptions) + ' or a color');
  },
  overlay: function overlay(value) {},
  underlay: function underlay(value) {},
  textCaptionStyles: function textCaptionStyles(value) {
    var fontFamily = value.fontFamily,
        fontSize = value.fontSize,
        fontWeight = value.fontWeight,
        fontStyle = value.fontStyle,
        textDecoration = value.textDecoration,
        textAlign = value.textAlign,
        stroke = value.stroke,
        letterSpacing = value.letterSpacing,
        lineSpacing = value.lineSpacing;

    invariantImage(fontFamily && fontSize && isNumber(fontSize), 'text caption', JSON.stringify(value), 'required options are \'fontFamily\' and \'fontSize\'', '/image_transformations#adding_text_captions');
    invariantImage((!fontWeight || includes(fontWeight, ['normal', 'bold'])) && (!fontStyle || includes(fontStyle, ['normal', 'italic'])) && (!textDecoration || includes(textDecoration, ['none', 'underline', 'strikethrough'])) && (!textAlign || includes(textAlign, ['left', 'center', 'right', 'start', 'end', 'justify'])) && (!stroke || includes(stroke, ['none', 'stroke'])) && (!letterSpacing || isNumber(letterSpacing)) && (!lineSpacing || isNumber(lineSpacing)), 'text caption', JSON.stringify(value), 'options are invalid', '/image_transformations#adding_text_captions');
  },
  defaultImage: function defaultImage(value) {
    return invariantImage(value.match(/^[\w+-]+\.[\w]{3,4}$/) && includes(value.substr(value.indexOf('.') + 1), formatOptions), 'defaultImage', value, 'must include a file extension which ' + shouldBeOneOf(formatOptions));
  },
  delay: function delay(value) {
    return invariantImage(isNumber(value) && value >= 0, 'delay', value, 'must be a positive number');
  },
  color: function color(value) {
    return invariantImage(value.match(colorRegex), 'color', value, 'must be a named color, short #rgb[a] or long #rrggbb[aa] hex value');
  },
  colorSpace: function colorSpace(value) {
    return invariantImage(includes(value, colorSpaceOptions) || value.match(/^icc:[\w+-]+\.[\w]{3,4}$/), 'colorSpace', value, shouldBeOneOf(colorSpaceOptions) + ' or \'icc:(public_id)\'');
  },
  dpr: function dpr(value) {
    return invariantImage(value === 'auto' || isNumber(value) && value > 0, 'dpr', value, 'should be `auto` or a number greater than 0');
  },
  page: function page(value) {
    return invariantImage(isNumber(value) && value > 0, 'page', value, 'should be an integer greater than 0');
  },
  density: function density(value) {
    return invariantImage(isNumber(value) && 0 < value && value <= 300, 'density', value, 'should be a number greater than 0 up to 300');
  },
  flags: function flags(value) {
    return invariantImage(value.split('.').every(function (flag) {
      return includes(flag, flagOptions);
    }), 'flags', JSON.stringify(value), shouldBeOneOf(flagOptions) + ', an array of options or \'.\' separated options');
  }
};

var urlParameters = {
  width: 'w_',
  height: 'h_',
  crop: 'c_',
  aspectRatio: 'ar_',
  gravity: 'g_',
  zoom: 'z_',
  x: 'x_',
  y: 'y_',
  fetchFormat: 'f_',
  quality: 'q_',
  radius: 'r_',
  angle: 'a_',
  effect: 'e_',
  opacity: 'o_',
  border: 'bo_',
  background: 'b_',
  overlay: 'l_',
  underlay: 'u_',
  defaultImage: 'd_',
  delay: 'dl_',
  color: 'co_',
  colorSpace: 'cs_',
  dpr: 'dpr_',
  page: 'pg_',
  density: 'dn_',
  flags: 'fl_'
};

function compileImageParameter(parameter, value) {
  if (value === null) {
    return '';
  }

  switch (parameter) {
    case 'flags':
      value = Array.isArray(value) ? value.join('.') : value;
      break;

    case 'overlay':
    case 'underlay':
      if (typeof value === 'object') {
        var stringStyle = value.publicId;
        if (!stringStyle) {
          {
            validate.textCaptionStyles(value);
          }
          var _value = value,
              letterSpacing = _value.letterSpacing,
              lineSpacing = _value.lineSpacing;

          stringStyle = [encodeURIComponent(value.fontFamily), value.fontSize, value.fontWeight, value.fontStyle, value.textDecoration, value.textAlign, value.stroke, letterSpacing ? 'letter_spacing_' + letterSpacing : 0, lineSpacing ? 'line_spacing_' + lineSpacing : 0].filter(function (option) {
            return option && option !== 'normal' && option !== 'none';
          }).join('_');
        }
        value = 'text:' + stringStyle + ':' + encodeURIComponent(value.text);
      }
      break;

    case 'border':
      if (typeof value === 'object') {
        {
          validate.borderObjectColor(value);
        }
        if (typeof value.width !== 'string' || value.width.substr(value.width.length - 2, 2) !== 'px') {
          value.width = (value.width || 1) + 'px';
        }
        value = value.width + '_solid_' + value.color;
      }
      value = value.toLowerCase();
    // falls through

    case 'color':
    case 'background':
      value = value.replace('#', 'rgb:');

    // no default
  }

  {
    if (!validate[parameter]) {
      throw new Error('Cloudinary Image :: unknown transform parameter provided: \'' + parameter + '\'');
    }
    validate[parameter](value);
  }

  return urlParameters[parameter] + value;
}

// import videoParameters from './parameters/video'

var index = urlBuilder({
  image: compileImageParameter
  // video: videoParameters,
});

return index;

})));
