var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import { invariant, includes, shouldBeOneOf } from './utils';

var typeOptions = ['upload', 'fetch', 'facebook', 'twitter', 'twitter_name', 'instagram', 'instagram_name', 'gplus', 'gravatar'];

export var compile = function compile(parameterSet, transform, defaultTransform) {
  if (!transform || !parameterSet || Object.keys(transform).length === 0) {
    return '';
  }

  var compile = function compile(parameters) {
    return Object.keys(parameters).map(function (param) {
      return parameterSet(param, parameters[param]);
    }).join(',');
  };

  return '/' + (Array.isArray(transform) ? transform.map(compile).join('/') : compile(_extends({}, defaultTransform, transform)));
};

var urlBuilder = function urlBuilder() {
  var parameterSets = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var defaultResourceType = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'image';
  return function (_ref) {
    var cloudName = _ref.cloudName,
        _ref$cdnSubdomain = _ref.cdnSubdomain,
        cdnSubdomain = _ref$cdnSubdomain === undefined ? false : _ref$cdnSubdomain,
        _ref$cname = _ref.cname,
        cname = _ref$cname === undefined ? 'res.cloudinary.com' : _ref$cname,
        _ref$secure = _ref.secure,
        defaultSecure = _ref$secure === undefined ? true : _ref$secure,
        _ref$defaults = _ref.defaults;
    _ref$defaults = _ref$defaults === undefined ? {} : _ref$defaults;

    var _ref$defaults$type = _ref$defaults.type,
        defaultType = _ref$defaults$type === undefined ? 'upload' : _ref$defaults$type,
        defaultTransform = _objectWithoutProperties(_ref$defaults, ['type']);

    process.env.NODE_ENV !== 'production' && invariant(cloudName, 'cloudName', cloudName, 'configuration is required', '/node_additional_topics#configuration_options');

    var baseUrl = cname + '/' + cloudName + '/';
    var sub = '';

    return function (publicId) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var _options$resourceType = options.resourceType,
          resourceType = _options$resourceType === undefined ? defaultResourceType : _options$resourceType,
          _options$secure = options.secure,
          secure = _options$secure === undefined ? defaultSecure : _options$secure,
          _options$type = options.type,
          type = _options$type === undefined ? defaultType : _options$type,
          _options$version = options.version,
          version = _options$version === undefined ? '1' : _options$version,
          transform = _objectWithoutProperties(options, ['resourceType', 'secure', 'type', 'version']);

      if (options.transform) {
        transform = options.transform;
      } else if (Array.isArray(options)) {
        transform = options;
      }

      process.env.NODE_ENV !== 'production' && invariant(!transform || Object.keys(transform).length === 0 || resourceType === 'raw' || includes(resourceType, Object.keys(parameterSets)), 'resourceType', resourceType, shouldBeOneOf(['raw'].concat(Object.keys(parameterSets))) + ', fix the resource type or add additional transform parameters to the configuration', null);

      process.env.NODE_ENV !== 'production' && invariant(includes(type, typeOptions), 'type', type, shouldBeOneOf(typeOptions), null);

      process.env.NODE_ENV !== 'production' && invariant(!cdnSubdomain || typeof cdnSubdomain === 'function', 'cdnSubdomain', cdnSubdomain, 'should be a CRC function: `(string) => number`', null);

      if (cdnSubdomain) {
        sub = 'a' + (cdnSubdomain(publicId) % 5 + 1) + '.';
      }

      transform = compile(parameterSets[resourceType], transform, defaultTransform);

      return 'http' + (secure ? 's' : '') + '://' + sub + baseUrl + resourceType + '/' + type + transform + '/v' + version + '/' + publicId;
    };
  };
};

export default urlBuilder;